/**
 * Copyright (c) Tiny Technologies, Inc. All rights reserved.
 * Licensed under the LGPL or a commercial license.
 * For LGPL see License.txt in the project root for license information.
 * For commercial licenses see https://7b2.com/
 *
 * Version: 5.2.2 (2020-04-23)
 */
(function () {
    var b2preview = (function () {
       'use strict';

        tinymce.PluginManager.add("b2preview", function (editor, url) {

            /*
            Add a custom icon to TinyMCE
             */
            editor.ui.registry.addIcon('b2-preview', '<svg width="24" height="24"><path d="M3.5 12.5c.5.8 1.1 1.6 1.8 2.3 2 2 4.2 3.2 6.7 3.2s4.7-1.2 6.7-3.2a16.2 16.2 0 0 0 2.1-2.8 15.7 15.7 0 0 0-2.1-2.8c-2-2-4.2-3.2-6.7-3.2a9.3 9.3 0 0 0-6.7 3.2A16.2 16.2 0 0 0 3.2 12c0 .2.2.3.3.5zm-2.4-1l.7-1.2L4 7.8C6.2 5.4 8.9 4 12 4c3 0 5.8 1.4 8.1 3.8a18.2 18.2 0 0 1 2.8 3.7v1l-.7 1.2-2.1 2.5c-2.3 2.4-5 3.8-8.1 3.8-3 0-5.8-1.4-8.1-3.8a18.2 18.2 0 0 1-2.8-3.7 1 1 0 0 1 0-1zm12-3.3a2 2 0 1 0 2.7 2.6 4 4 0 1 1-2.6-2.6z" fill-rule="nonzero"></path></svg>');

            /*
            Use to store the instance of the Dialog
             */
            var _dialog = false;

            /*
            An array of options to appear in the "Type" select box.
             */
            var _typeOptions = [];

            /**
             * Get the Dialog Configuration Object
             *
             * @returns {{buttons: *[], onSubmit: onSubmit, title: string, body: {}}}
             * @private
             */
            function _getDialogConfig(data)
            {
                return {
                    title: 'Preview',
                    size: "large",
                    body: {
                        type: 'panel',
                        items: [{
                        type: 'htmlpanel',
                        html: '<div id="b2-preview" class="entry-content"></div>'
                        }]
                    },
                    buttons: [{
                        type: "cancel",
                        name: "close",
                        text: "Close"
                    }]
                };
            }

            function _onAction()
            {
                // Open a Dialog, and update the dialog instance var
                _dialog = editor.windowManager.open(_getDialogConfig());

                // block the Dialog, and commence the data update
                // Message is used for accessibility
                _dialog.block('Loading...');

                writeEditor.preview(function(data){
                    document.querySelector('#b2-preview').innerHTML = data
                    // unblock the dialog
                    _dialog.unblock();
                })
            }

            // Define the Toolbar button
            editor.ui.registry.addButton('b2preview', {
                tooltip: "Preview",
                icon: 'b2-preview',
                onAction: _onAction
            });

            // Return details to be displayed in TinyMCE's "Help" plugin, if you use it
            // This is optional.
            return {
                getMetadata: function () {
                    return {
                        name: "B2主题预览插件",
                        url: "https://7b2.com"
                    };
                }
            };
        });
    }());
})();